@@ -284,12 +284,12 @@ get_uni_me <- function(x) {
 }
 
 # save all me-terms within a tidy data.frame
-tidy_meef <- function(bterms, data, old_levels = NULL) {
-  uni_me <- get_uni_me(bterms)
+frame_me <- function(x, data, old_levels = NULL) {
+  uni_me <- get_uni_me(x)
   if (!length(uni_me)) {
-    return(empty_meef())
+    return(empty_meframe())
   }
-  if (has_subset(bterms)) {
+  if (has_subset(x)) {
     # 'Xme' variables need to be the same across univariate models
     stop2("Argument 'subset' is not supported when using 'me' terms.")
   }
@@ -297,41 +297,46 @@ tidy_meef <- function(bterms, data, old_levels = NULL) {
     term = uni_me, xname = "", grname = "",
     stringsAsFactors = FALSE
   )
-  levels <- vector("list", nrow(out))
+  levels <- list()
   for (i in seq_rows(out)) {
     tmp <- eval2(out$term[i])
     out$xname[i] <- tmp$term
     if (isTRUE(nzchar(tmp$gr))) {
       out$grname[i] <- tmp$gr
-      if (length(old_levels)) {
-        levels[[i]] <- old_levels[[tmp$gr]]
-      } else {
-        levels[[i]] <- extract_levels(get(tmp$gr, data))
+      if (is.null(levels[[tmp$gr]])) {
+        levels[[tmp$gr]] <- extract_levels(get(tmp$gr, data))
       }
     }
   }
   out$coef <- rename(paste0("me", out$xname))
-  out$cor <- isTRUE(bterms$mecor)
-  names(levels) <- out$grname
-  levels <- levels[lengths(levels) > 0L]
-  if (length(levels)) {
-    levels <- levels[!duplicated(names(levels))]
-    attr(out, "levels") <- levels
+  out$cor <- isTRUE(x$mecor)
+  if (!is.null(old_levels)) {
+    # for newdata numeration has to depend on the original levels
+    set_levels(out) <- old_levels
+    set_levels(out, "used") <- levels
+  } else {
+    set_levels(out) <- levels
   }
-  structure(out, class = c("meef_frame", "data.frame"))
+  class(out) <- meframe_class()
+  out
 }
 
-empty_meef <- function() {
+empty_meframe <- function() {
   out <- data.frame(
     term = character(0), xname = character(0),
     grname = character(0), cor = logical(0),
     stringsAsFactors = FALSE
   )
-  structure(out, class = c("meef_frame", "data.frame"))
+  class(out) <- meframe_class()
+  out
 }
 
-is.meef_frame <- function(x) {
-  inherits(x, "meef_frame")
+meframe_class <- function() {
+  c("meframe", "data.frame")
+}
+
+is.meframe <- function(x) {
+  inherits(x, "meframe")
 }
 
 # handle default of correlations between 'me' terms
@@ -350,7 +355,7 @@ get_sp_vars <- function(x, type) {
 # @param data data frame containing the monotonic variables
 # @return a data.frame with one row per special term
 # TODO: refactor to store in long format to avoid several list columns?
-tidy_spef <- function(x, data) {
+frame_sp <- function(x, data) {
   if (is.formula(x)) {
     x <- brmsterms(x, check_response = FALSE)$dpars$mu
   }
@@ -360,6 +365,15 @@ tidy_spef <- function(x, data) {
   }
   mm <- sp_model_matrix(form, data, rename = FALSE)
   out <- data.frame(term = colnames(mm), stringsAsFactors = FALSE)
+
+  # fixes issue #1754
+  try_parse <- try(parse(text = out$term))
+  if (is_try_error(try_parse)) {
+    stop2("Cannot parse special effects terms (see above error message). ",
+          "Likely this is because of factor levels with special symbols ",
+          "such as '$' or '@'.")
+  }
+
   out$coef <- rename(out$term)
   calls_cols <- c(paste0("calls_", all_sp_types()), "joint_call")
   list_cols <- c("vars_mi", "idx_mi", "idx2_mi", "ids_mo", "Imo")
@@ -439,24 +453,33 @@ tidy_spef <- function(x, data) {
 
   # extract information on covariates
   # only non-zero covariates are relevant to consider
-  not_one <- apply(mm, 2, function(x) any(x != 1))
-  cumsum_not_one <- cumsum(not_one)
-  out$Ic <- ifelse(not_one, cumsum_not_one, 0)
+  has_covars <- attr(mm, "covars")
+  cumsum_covars <- cumsum(has_covars)
+  out$Ic <- ifelse(has_covars, cumsum_covars, 0)
+  class(out) <- spframe_class()
   out
 }
 
+spframe_class <- function() {
+  c("spframe", "data.frame")
+}
+
+is.spframe <- function(x) {
+  inherits(x, "spframe")
+}
+
 # extract names of monotonic simplex parameters
-# @param spef output of tidy_spef
+# @param spframe output of frame_sp
 # @param use_id use the 'id' argument to construct simo labels?
-# @return a character vector of length nrow(spef)
-get_simo_labels <- function(spef, use_id = FALSE) {
-  out <- named_list(spef$term)
-  I <- which(lengths(spef$Imo) > 0)
+# @return a character vector of length nrow(spframe)
+get_simo_labels <- function(spframe, use_id = FALSE) {
+  out <- named_list(spframe$term)
+  I <- which(lengths(spframe$Imo) > 0)
   for (i in I) {
     # use the ID as label if specified
     out[[i]] <- ifelse(
-      use_id & !is.na(spef$ids_mo[[i]]), spef$ids_mo[[i]],
-      paste0(spef$coef[i], seq_along(spef$Imo[[i]]))
+      use_id & !is.na(spframe$ids_mo[[i]]), spframe$ids_mo[[i]],
+      paste0(spframe$coef[i], seq_along(spframe$Imo[[i]]))
     )
   }
   unlist(out)
@@ -482,8 +505,8 @@ get_sdy <- function(x, data = NULL) {
   sdy
 }
 
-# names of grouping variables used in measurement error terms
-get_me_groups <- function(x) {
+# get names of grouping variables from me terms
+get_me_group_vars <- function(x) {
   uni_me <- get_uni_me(x)
   out <- lapply(uni_me, eval2)
   out <- ufrom_list(out, "gr")
@@ -506,7 +529,7 @@ sp_model_matrix <- function(formula, data, types = all_sp_types(), ...) {
   terms_replace <- terms_unique[grepl_expr(regex, terms_unique)]
   dummies <- paste0("dummy", seq_along(terms_replace), "__")
   data[dummies] <- list(1)
-  terms_comb <- rep(NA, length(terms_split))
+  terms_comb <- covars <- rep(NA, length(terms_split))
   # loop over terms and add dummy variables
   for (i in seq_along(terms_split)) {
     replace_i <- grepl_expr(regex, terms_split[[i]])
@@ -514,6 +537,8 @@ sp_model_matrix <- function(formula, data, types = all_sp_types(), ...) {
     dummies_i <- dummies[match(terms_i_replace, terms_replace)]
     terms_split[[i]][replace_i] <- dummies_i
     terms_comb[i] <- paste0(terms_split[[i]], collapse = ":")
+    # non-special covariates are part of the term
+    covars[i] <- !all(replace_i)
   }
   new_formula <- str2formula(terms_comb)
   attributes(new_formula) <- attributes(formula)
@@ -522,6 +547,7 @@ sp_model_matrix <- function(formula, data, types = all_sp_types(), ...) {
   colnames(out) <- rm_wsp(colnames(out))
   # recover original column names
   colnames(out) <- rename(colnames(out), dummies, terms_replace)
+  attr(out, "covars") <- covars
   out
 }
 
